#ifndef __TWINVQ_H_
#define __TWINVQ_H_

#include <stdio.h> /* for BUFSIZ definition */

enum INIT_ERROR_CODE {
  TVQ_NO_ERROR = 0,
  TVQ_ERROR,
  TVQ_ERROR_VERSION,
  TVQ_ERROR_CHANNEL,
  TVQ_ERROR_MODE,
  TVQ_ERROR_PARAM,
  TVQ_ERROR_N_CAN
};
#define TVQ_UNKNOWN_VERSION	-1

#define V2			0
#define V2PP			1
#define N_VERSIONS		2

/* window types */
enum WINDOW_TYPE {
  ONLY_LONG_WINDOW = 0,
  LONG_SHORT_WINDOW,
  ONLY_SHORT_WINDOW,
  SHORT_LONG_WINDOW,
  SHORT_MEDIUM_WINDOW,
  MEDIUM_LONG_WINDOW,
  LONG_MEDIUM_WINDOW,
  MEDIUM_SHORT_WINDOW,
  ONLY_MEDIUM_WINDOW,
};

/*
 * block types
*/
enum BLOCK_TYPE {
  BLK_SHORT = 0,
  BLK_MEDIUM,
  BLK_LONG,
  BLK_PPC,
};
#define N_BTYPE		3	/* number of block types */
#define N_INTR_TYPE	4	/* number of interleave types*/
#define N_CH_MAX	2


/** Type definition of code information interface  **/
typedef struct {
  int	w_type;			/* block type */
  int	btype;
  int*	segment_sw[N_CH_MAX];	/* FBC info */
  int*	band_sw[N_CH_MAX];
  int*	fg_intensity[N_CH_MAX];
  int*	wvq;			/* VQ info */
  int*	fw;			/* BSE info */
  int*	fw_alf;
  int*	pow;			/* gain info */
  int*	lsp[N_CH_MAX];		/* LSP info */
  int	pit[N_CH_MAX];		/* PPC info */
  int*	pls;
  int	pgain[N_CH_MAX];
  int*	bc[N_CH_MAX];		/* EBC info */
  void*	manager;

}	INDEX;

/* type definition of tvqConfInfoSubBlock */
typedef struct {
  int	sf_sz;		/* subframe size */
  int	nsf;		/* number of subframes */
  int	ndiv;		/* number of division of weighted interleave vector */
  int	ncrb;		/* number of Bark-scale subbands */
  int	fw_ndiv;	/* number of division of BSE VQ */
  int	fw_nbit;	/* number of bits for BSE VQ */
  int	nsubg;		/* number of sub-blocks for gain coding */
  int	ppc_enable;	/* PPC switch */
  int	ebc_enable;	/* EBC switch */
  int	ebc_crb_base;	/* EBC base band */
  int	ebc_bits;	/* EBC bits */
  int	fbc_enable;	/* FBC switch */
  int	fbc_n_segment;	/* FBC number of segments */
  int	fbc_nband;	/* FBC number of subbands */
  int*	fbc_crb_tbl;	/* FBC subband table */

}	tvqConfInfoSubBlock;

/* type definition of tvqConfInfo */
typedef struct {
  int			N_CH;		/* frame configuration */
  int			BITS_WTYPE;	/* window type coding */
  int			LSP_BIT0;	/* LSP coding */
  int			LSP_BIT1;
  int			LSP_BIT2;
  int			LSP_SPLIT;
  int			FW_ARSW_BITS;	/* Bark-scale envelope coding */
  int			GAIN_BITS;	/* gain coding */
  int			SUB_GAIN_BITS;
  int			N_DIV_P;	/* pitch excitation */
  int			BASF_BIT;
  int			PGAIN_BIT;
  tvqConfInfoSubBlock	cfg[N_BTYPE];  /* block type dependent parameters */

}			tvqConfInfo;


/*** Definitions about TwinVQ bitstream header ***/
#define KEYWORD_BYTES   4
#define VERSION_BYTES   8
#define ELEM_BYTES      sizeof(unsigned long)

typedef struct{
  char		ID[KEYWORD_BYTES+VERSION_BYTES+1];
  int		size;
  int		channelMode;   /* channel mode (mono:0/stereo:1) */
  int		bitRate;       /* bit rate (kbit/s) */
  int		samplingRate;  /* sampling rate (44.1 kHz -> 44) */
  int		securityLevel; /* security level (always 0) */
  char		Name[BUFSIZ];
  char		Comt[BUFSIZ];
  char		Auth[BUFSIZ];
  char		Cpyr[BUFSIZ];
  char		File[BUFSIZ];
  char		Extr[BUFSIZ];  // add by OKAMOTO 99.12.21
  int		Dsiz;

}		headerInfo;


#endif /*  __TWINVQ_H_ */
